#!/usr/bin/python

## code is made by Pim Muilwijk 
## little adjustment for Domoticz by Albert @ Hakvoort (dot) co
## version 0.3

import datetime, time, threading, serial, socket

class ZehnderWHR930:
    def __init__(self, port):
        
        self.port               = port                  # Port of the USB to RS-232 converter
        self.timeout            = 10                    # Time-out of the serial connection in seconds
        self.active             = True                  # Whether the instance is running
        self.lock               = False                 # Prevents the GetStatus function from running while a socket is connected
        self.outsideAirTemp     = False
        self.supplyAirTemp      = False
        self.returnAirTemp      = False
        self.exhaustAirTemp     = False
        self.returnAirLevel     = False
        self.supplyAirLevel     = False
        self.fanLevel           = False
        self.lowTime            = False
        self.mediumTime         = False
        self.highTime           = False
        self.filterTime         = False
        self.lastUpdated        = datetime.datetime.now()

        self.updateThread       = threading.Thread()
        self.timerThread        = threading.Thread(target = self.Timer)
        self.timerThread.start()
        
    def Timer(self):
        while self.active:
            if datetime.datetime.now().second < 55:
                time.sleep(55 - datetime.datetime.now().second - datetime.datetime.now().microsecond / 1000000.0)
            else:
                time.sleep(115 - datetime.datetime.now().second - datetime.datetime.now().microsecond / 1000000.0)
            
            # Wait for possible socket connections to disconnect
            while self.lock:
                pass
            
            self.updateThread = threading.Thread(target = self.GetVentilationData)
            self.updateThread.start()

    def SendSerialCommand(self, command, ackBytes):
        response = []
        
        serialConnection = serial.Serial(port = self.port, baudrate = 9600, bytesize = serial.EIGHTBITS, parity = serial.PARITY_NONE, stopbits = serial.STOPBITS_ONE, timeout = self.timeout)
        serialConnection.write(command)

        start = time.time()

        # Wait for response to be acquired or the connection to time out
        while response[-2:] != ackBytes and time.time() - start < self.timeout:
            response.append(serialConnection.read().encode('hex'))


#	print self.outsideAirTemp
        serialConnection.close()
        
        return response

    def GetVentilationData(self):
        # Obtain temperatures
        raw = self.SendSerialCommand('\x07\xF0\x00\x0F\x00\xBC\x07\x0F', ['07', '0f'])

        if raw[-2:] == ['07', '0f']:
            self.outsideAirTemp  = int(raw[7], 16) / 2.0 - 20
            self.supplyAirTemp   = int(raw[8], 16) / 2.0 - 20
            self.returnAirTemp   = int(raw[9], 16) / 2.0 - 20
            self.exhaustAirTemp  = int(raw[10], 16) / 2.0 - 20
        else:
            self.outsideAirTemp  = self.supplyAirTemp = self.returnAirTemp = self.exhaustAirTemp = False

        # Obtain levels
        raw = self.SendSerialCommand('\x07\xF0\x00\xCD\x00\x7A\x07\x0F', ['07', '0f'])

        if raw[-2:] == ['07', '0f']:
            self.returnAirLevel = int(raw[13], 16)
            self.supplyAirLevel = int(raw[14], 16)
            fanLevel            = int(raw[15], 16) - 1
        else:
            self.returnAirLevel = self.supplyAirLevel = self.fanLevel = False

        if fanLevel != self.fanLevel and self.fanLevel != False:
            self.fanLevel = fanLevel
        
        # Obtain operating times
        raw = self.SendSerialCommand('\x07\xF0\x00\xDD\x00\x8A\x07\x0F', ['07', '0f'])

        if raw[-2:] == ['07', '0f']:
            self.lowTime     = int(''.join(raw[10:13]), 16)
            self.mediumTime  = int(''.join(raw[13:16]), 16)
            self.highTime    = int(''.join(raw[24:27]), 16)
            self.filterTime  = int(''.join(raw[22:24]), 16)
        else:
            self.lowTime     = self.mediumTime = self.highTime = self.filterTime = False

        self.lastUpdated = datetime.datetime.now()

    def SetFanLevel(self, fanLevel):
        if fanLevel == 0:
            raw = self.SendSerialCommand('\x07\xF0\x00\x99\x01\x00\x47\x07\x0F', ['07', 'f3'])
        if fanLevel == 1:
            raw = self.SendSerialCommand('\x07\xF0\x00\x99\x01\x01\x48\x07\x0F', ['07', 'f3'])
        if fanLevel == 2:
            raw = self.SendSerialCommand('\x07\xF0\x00\x99\x01\x02\x49\x07\x0F', ['07', 'f3'])
        elif fanLevel == 3:
            raw = self.SendSerialCommand('\x07\xF0\x00\x99\x01\x03\x4A\x07\x0F', ['07', 'f3'])
        elif fanLevel == 4:
            raw = self.SendSerialCommand('\x07\xF0\x00\x99\x01\x04\x4B\x07\x0F', ['07', 'f3'])

        if raw == ['07', 'f3']:
            self.fanLevel       = fanLevel
            self.lastUpdated    = datetime.datetime.now()
        else:
            self.fanLevel = False

        return self.fanLevel

# Create an instance of the ZehnderWHR930 class
Ventilation = ZehnderWHR930('/dev/YPort')

# Starting socket server for receiving commands
sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
sock.bind(('', 7788))
sock.listen(1)

while Ventilation.active:
    connection, clientAddress = sock.accept()
    data = connection.recv(128).strip().split('||')

    Ventilation.lock = True

    # Wait for possible GetVentilationData thread to join
    if Ventilation.updateThread.is_alive():
        Ventilation.updateThread.join()
    
    if data[1] == 'SetLevel':
        fanLevel = Ventilation.SetFanLevel(int(data[2]))

        connection.sendall('{0}'.format(fanLevel))
    elif data[1] == 'GetData':
        connection.sendall('{0}|{1}|{2}|{3}|{4}|{5}|{6}|{7}|{8}|{9}|{10}|{11}'.format(int(time.mktime(Ventilation.lastUpdated.timetuple())), Ventilation.outsideAirTemp, Ventilation.supplyAirTemp, Ventilation.returnAirTemp, Ventilation.exhaustAirTemp, Ventilation.returnAirLevel, Ventilation.supplyAirLevel, Ventilation.fanLevel, Ventilation.lowTime, Ventilation.mediumTime, Ventilation.highTime, Ventilation.filterTime))
    elif data[1] == 'SHUTDOWN':
        Ventilation.active = False
    
    connection.close()
    
    Ventilation.lock = False

print datetime.datetime.now()
print 'Shutting down...'

Ventilation.updateThread.join()
Ventilation.timerThread.join()

sock.shutdown(2)
sock.close()

